import unittest

from cad_parser.definitions import Patient

class TestPatient(unittest.TestCase):
    def test_constructor(self):
        patient = Patient(1, 'Mustermann', 'Max')

        self.assertEqual(patient.id, 1)
        self.assertEqual(patient.last_name, 'Mustermann')
        self.assertEqual(patient.first_name, 'Max')
        