import unittest

from cad_parser.definitions import Tooth

class TestTooth(unittest.TestCase):
    def test_constructor(self):
        tooth = Tooth(fdi=1, color='A1', implant_system=None, implant_type=None, is_anatomical=None, type='AnatomicCrown')

        self.assertEqual(tooth.fdi, 1)
        self.assertEqual(tooth.color, 'A1')
        self.assertEqual(tooth.implant_type, None)
        self.assertEqual(tooth.is_anatomical, None)
        self.assertEqual(tooth.type, 'AnatomicCrown')
